@@ -310,9 +310,9 @@ def compare_data(xparray, simarray, xspairlist, vardata=False):
         #code.interact(local=locals())
 
         objout += objarray.sum()
-        print "OBJOUT(%d,%d):%f  |\t\tOBJOUT(CUM):%f"%(xparrayaxis, simarrayaxis, objarray.sum(), objout)
+        #print "OBJOUT(%d,%d):%f  |\t\tOBJOUT(CUM):%f"%(xparrayaxis, simarrayaxis, objarray.sum(), objout)
 
-    print "OBJOUT(total):", objout
+    #print "OBJOUT(total):", objout
     return objout
 
 def getgenparambounds(params, omag=1, N=1000., useparams=[], usemag=None, useN=None ):
