@@ -1,10 +1,14 @@
-from pysb import *
+"""
+EARM 1.0 (extrinsic apoptosis reaction model)
 
-# Albeck JG, Burke JM, Spencer SL, Lauffenburger DA, Sorger PK, 2008
-# Modeling a Snap-Action, Variable-Delay Switch Controlling Extrinsic
-# Cell Death. PLoS Biol 6(12): e299. doi:10.1371/journal.pbio.0060299
-#
-# http://www.plosbiology.org/article/info:doi/10.1371/journal.pbio.0060299
+Albeck JG, Burke JM, Spencer SL, Lauffenburger DA, Sorger PK, 2008
+Modeling a Snap-Action, Variable-Delay Switch Controlling Extrinsic
+Cell Death. PLoS Biol 6(12): e299. doi:10.1371/journal.pbio.0060299
+
+http://www.plosbiology.org/article/info:doi/10.1371/journal.pbio.0060299
+"""
+
+from pysb import *
 
 Model()
 
@@ -261,13 +265,13 @@ inhibit(cSmac, XIAP, kf28, kr28)
 
 
 # Fig 4B
-Observe('Bid',   Bid(b=None))
-Observe('PARP',  PARP(b=None))
-Observe('mSmac', mSmac(b=None))
+Observable('Bid_unbound',   Bid(b=None))
+Observable('PARP_unbound',  PARP(b=None))
+Observable('mSmac_unbound', mSmac(b=None))
 # this is what I originally thought 4B was actually plotting
-Observe('tBid',  tBid())
-Observe('CPARP', CPARP())
-Observe('cSmac', cSmac())
+Observable('tBid_total',  tBid())
+Observable('CPARP_total', CPARP())
+Observable('cSmac_total', cSmac())
 
 
 
