@@ -1,6 +1,8 @@
 import inspect
 from pysb import *
 import pysb.core
+from pysb.core import ComponentSet
+import numbers
 
 DEFAULT_UNI_KF = 1e6 # In units of sec^-1
 DEFAULT_BI_KF = 1e-4 # In units of nM^-1 sec^-1
@@ -16,6 +18,12 @@ def alias_model_components(model=None):
     caller_globals.update(components)
 
 
+def monomer_pattern_label(mp):
+    """Return a reasonable string label for a MonomerPattern."""
+    site_values = [str(x) for x in mp.site_conditions.values() if x is not None]
+    return mp.monomer.name + ''.join(site_values)
+
+
 def two_step_conv(Sub1, Sub2, Prod, klist, site='bf'):
     """Automation of the Sub1 + Sub2 <> Sub1:Sub2 >> Prod two-step reaction (i.e. dimerization).
     This function assumes that there is a site named 'bf' (bind site for fxn)
@@ -225,84 +233,167 @@ def simple_bind_table(bindtable, parmlist, lmodel, site='bf'):
         print "WARNING, unassigned parameters from list", parmlist
         print "Assigned",pc,"parameter pairs from a total of", len(parmlist)
 
-#-------------------------------------------------------------------------
-# John B Macro Funcs
-#-------------------------------------------------------------------------
+def catalyze(enzyme, e_site, substrate, s_site, product, klist):
+    """Generate the two-step catalytic reaction E + S <> E:S >> E + P.
 
-def two_step_mod_sites(enz, enz_site, sub, sub_site, prod, klist=None):
-    """Automation of the Enz + Sub <> Enz:Sub >> Enz + Prod two-step catalytic reaction.
-    This function assumes that there is a site named 'bf' (bind site for fxn)
-    which it uses by default. This also assumes Enz returns to its original state.
-    In an aim for simplicity, site 'bf' need not be passed when calling the function by
-    the reactants, but THE FULL STATE OF THE PRODUCT must be specified"""
+    Parameters
+    ----------
+    enzyme, substrate, product : Monomer or MonomerPattern
+        E, S and P in the above reaction.
+    e_site, s_site : string
+        The names of the sites on `enzyme` and `substrate` (respectively) where
+        they bind each other to form the E:S complex.
+    klist : list of 3 Parameters or list of 3 numbers
+        Forward, reverse and catalytic rate constants (in that order). If
+        Parameters are passed, they will be used directly in the generated
+        Rules. If numbers are passed, Parameters will be created with
+        automatically generated names based on the names and states of enzyme,
+        substrate and product and these parameters will be included at the end
+        of the returned component list.
+
+    Returns
+    -------
+    components : ComponentSet
+        The generated components. Contains two Rules (bidirectional complex
+        formation and unidirectional product dissociation), and optionally three
+        Parameters if klist was given as plain numbers.
+
+    Notes
+    -----
+    When passing a MonomerPattern for `enzyme` or `substrate`, do not include
+    `e_site` or `s_site` in the respective patterns. The macro will handle this.
+
+    Examples
+    --------
+    Using distinct Monomers for substrate and product::
+
+        Model()
+        Monomer('E', ['b'])
+        Monomer('S', ['b'])
+        Monomer('P')
+        catalyze(E, 'b', S, 'b', P, (1e-4, 1e-1, 1))
+
+    Using a single Monomer for substrate and product with a state change::
+
+        Monomer('Kinase', ['b'])
+        Monomer('Substrate', ['b', 'y'], {'y': ('U', 'P')})
+        catalyze(Kinase, 'b', Substrate(y='U'), 'b', Substrate(y='P'),
+                 (1e-4, 1e-1, 1))
+
+    """
     
+    # turn any Monomers into MonomerPatterns
+    substrate = substrate()
+    enzyme = enzyme()
+    product = product()
+
+    # verify that sites are valid
+    if e_site not in enzyme.monomer.sites_dict:
+        raise ValueError("e_site '%s' not present in monomer '%s'" %
+                         (e_site, enzyme.monomer.name))
+    if s_site not in substrate.monomer.sites_dict:
+        raise ValueError("s_site '%s' not present in monomer '%s'" %
+                         (s_site, substrate.monomer.name))
+
+    # generate the rule names
     # FIXME: this will fail if the argument passed is a Complex object. 
-    r1_name = 'cplx_%s%s_%s%s' % (sub.monomer.name, ''.join(filter(lambda a: a != None, sub.site_conditions.values())),
-                                     enz.monomer.name, ''.join(filter(lambda a: a != None, enz.site_conditions.values())))
-    r2_name = 'diss_%s%s_via_%s%s' % (prod.monomer.name, ''.join(filter(lambda a: a != None, prod.site_conditions.values())),
-                                         enz.monomer.name, ''.join(filter(lambda a: a != None, enz.site_conditions.values())))
+    substrate_name = monomer_pattern_label(substrate)
+    enzyme_name = monomer_pattern_label(enzyme)
+    product_name = monomer_pattern_label(product)
+    rc_name = 'complex_%s_%s' % (substrate_name, enzyme_name)
+    rd_name = 'dissociate_%s_from_%s' % (product_name, enzyme_name)
 
-    # Default names for the parameters
-    if (not klist):
-        kf = Parameter(r1_name + '_kf', DEFAULT_BI_KF)
-        kr = Parameter(r1_name + '_kr', DEFAULT_KR)
-        kc = Parameter(r2_name + '_kc', DEFAULT_KC)
+    # set up some aliases to the patterns we'll use in the rules
+    enzyme_free = enzyme({e_site: None})
+    substrate_free = substrate({s_site: None})
+    es_complex = enzyme({e_site: 1}) % substrate({s_site: 1})
+    # if product is actually a variant of substrate, we need to explicitly say
+    # that it is no longer bound to enzyme
+    if product.monomer is substrate.monomer:
+        product = product({e_site: None})
+
+    if all(isinstance(x, Parameter) for x in klist):
+        kf, kr, kc = klist
+        params_created = False
+    elif all(isinstance(x, numbers.Real) for x in klist):
+        # if klist is numbers, generate the Parameters
+        kf = Parameter(rc_name + '_kf', klist[0])
+        kr = Parameter(rc_name + '_kr', klist[1])
+        kc = Parameter(rd_name + '_kc', klist[2])
+        params_created = True
     else:
-        kf, kr, kc = klist #forward, reverse, catalytic
+        raise ValueError("klist must contain Parameters objects or numbers")
      
-    assert enz_site in enz.monomer.sites_dict, \
-        "Required site %s not present in %s as required"%(enz_site, Enz.monomer.name)
-    assert sub_site in sub.monomer.sites_dict, \
-        "Required site %s not present in %s as required"%(sub_site, Sub.monomer.name)
-    #assert prod.is_concrete(), \
-    #    "The product %s is not a fully specified (concrete) pattern." % (prod) 
+    # create the rules
+    rc = Rule(rc_name, enzyme_free + substrate_free <> es_complex, kf, kr)
+    rd = Rule(rd_name, es_complex >> enzyme_free + product, kc)
 
-    # make the intermediate complex components
-    etmpdict = enz.site_conditions.copy()
-    stmpdict = sub.site_conditions.copy()
+    # build a set of components that were created
+    components = ComponentSet([rc, rd])
+    if params_created:
+        components |= ComponentSet([kf, kr, kc])
+
+    return components
+
+def one_step_mod(enz, sub, prod, kf=None):
+    """Automation of the Enz + Sub >> Enz + Prod one-step catalytic
+    reaction. Assumes state of Enz is unchanged.
+
+    enz, sub and prod are MonomerPatterns.
+    kf is a Parameter object.
+    """
+
+    r_name = 'cat_%s%s_%s%s' % (sub.monomer.name,
+            ''.join(filter(lambda a: a != None, sub.site_conditions.values())),
+            enz.monomer.name,
+            ''.join(filter(lambda a: a != None, enz.site_conditions.values())))
+
+    # If no parameters are provided, create defaults
+    if (not kf):
+        kf = Parameter(r1_name + '_kf', DEFAULT_UNI_KF)
+     
+    # Write the rule
+    r = Rule(r_name, enz + sub <> enz_cplx % sub_cplx, kf, kr)
     
-    etmpdict[enz_site] = 1
-    stmpdict[sub_site] = 1
+    # Return the components created by this function
+    if not kf:
+        components_created = [r, kf]
+    else:
+        components_created = [r]
 
-    enz_cplx = enz.monomer(etmpdict)
-    sub_cplx = sub.monomer(stmpdict)
-
-    # add the 'bf' site to the patterns
-    enz.site_conditions[enz_site] = None
-    sub.site_conditions[sub_site] = None
-
-    # now that we have the complex elements formed we can write the first step rule
-    Rule(r1_name, enz + sub <> enz_cplx % sub_cplx, kf, kr)
-    
-    # and finally the rule for the catalytic transformation
-    Rule(r2_name, enz_cplx % sub_cplx >> enz + prod, kc)
-
-def two_step_mod(Enz, Sub, Prod, klist=None, site='bf'):
-    two_step_mod_sites(Enz, site, Sub, site, Prod, klist)
+    return components_created
 
 
 def bind(Sub1, site1, Sub2, site2, klist=None):
-    """Automation of the Sub1 + Sub2 <> Sub1:Sub2 one-step complex formation, but allows
-    the binding sites of both species to be specified.
-    Note that it expects Sub1 and Sub2 to be MonomerPatterns (not Monomers), and site1
-    and site2 strings indicating the names of the binding sites.
+    """Automation of the Sub1 + Sub2 <> Sub1:Sub2 one-step complex formation,
+    but allows the binding sites of both species to be specified. Note that it
+    expects Sub1 and Sub2 to be MonomerPatterns (not Monomers), and site1
+    and site2 to be strings indicating the names of the binding sites.
     klist is list of Parameter objects. If klist is not provided as an argument,
     the bind function will generate default forward and reverse parameters.
     """
 
-    # FIXME: this will fail if the argument passed is a complex, or a Monomer object... 
-    r_name = 'cplx_%s%s_%s%s' % (Sub1.monomer.name, ''.join(filter(lambda a: a != None, Sub1.site_conditions.values())),
-                                     Sub2.monomer.name, ''.join(filter(lambda a: a != None, Sub2.site_conditions.values())))
+    # FIXME: this will fail if the argument passed is a complex, or a Monomer
+    # object... 
+    r_name = 'cplx_%s%s_%s%s' % (Sub1.monomer.name,
+             ''.join(filter(lambda a: a != None,
+             Sub1.site_conditions.values())), Sub2.monomer.name,
+             ''.join(filter(lambda a: a != None,
+             Sub2.site_conditions.values())))
     
     assert site1 in Sub1.monomer.sites_dict, \
-        "Required site %s not present in %s as required"%(site1, Sub1.monomer.name)
+        "Required site %s not present in %s as required" % \
+        (site1, Sub1.monomer.name)
     assert site2 in Sub2.monomer.sites_dict, \
-        "Required site %s not present in %s as required"%(site2, Sub2.monomer.name)
+        "Required site %s not present in %s as required" % \
+        (site2, Sub2.monomer.name)
 
     if (not klist):
         # Default parameter values
-        # Diffusion limited on rate of 1e6 and offrate of 1e-1 implies 100nM binding
-        # FIXME get rid of these magic numbers!!!! Put in a global setting of some kind
+        # Diffusion limited on rate of 1e6 and offrate of 1e-1 implies
+        # 100nM binding
+        # FIXME get rid of these magic numbers!!!! Put in a global setting
+        # of some kind
         kf = Parameter(r_name + '_kf', DEFAULT_BI_KF)
         kr = Parameter(r_name + '_kr', DEFAULT_KR)
     else: 
@@ -318,12 +409,11 @@ def bind(Sub1, site1, Sub2, site2, klist=None):
     Sub1Cplx = Sub1.monomer(s1tmpdict)
     Sub2Cplx = Sub2.monomer(s2tmpdict)
 
-    # create the sites for the monomers
+    # Create the sites for the monomers
     Sub1.site_conditions[site1] = None
     Sub2.site_conditions[site2] = None
-    # now that we have the complex elements formed we can write the first step rule
+    # Now that we have the complex elements formed we can write the rule
     Rule(r_name, Sub1 + Sub2 <> Sub1Cplx % Sub2Cplx, kf, kr)
-   
  
 def simple_bind(Sub1, Sub2, klist, site='bf'):
     """Automation of the Sub1 + Sub2 <> Sub1:Sub2 one-step complex formation. 
@@ -403,6 +493,41 @@ def multisite_bind_table(bindtable):
                 pc += 1
                 rc += 1
 
+def bind_table(bindtable, row_site, col_site):
+    """This assumes that the monomers passed are in their desired state without
+    the sites which will be used for binding.
+    bindtable is a list of lists denoting the reactions between two types of reactants
+    as follows:
+
+    bindtable[0]: [                     reactypeA0,  reactypeA1, ...,  reactypeAN]
+    bindtable[1]: [reactypeB0,  (fwdrate, revrate),            , ...,            ]
+    bindtable[2]: [reactypeB1,                 ,               , ...,            ]
+
+    To indicate that no interaction occurs, simply enter None in the bind table.
+
+    """
+
+    # parse the list, extract reactants, products and parameter families
+    #first line is one set of reactants
+    react_cols = bindtable[0]
+    react_rows = [row[0] for row in bindtable[1:]]
+
+    # Notice this makes intrxns indexed by intrxns[row][col]
+    intrxns = [row[1:] for row in bindtable[1:]]
+
+    # loop over interactions
+    pc = 1 # parameter counter
+    rc = 1 # rule counter, easy way of making sure names don't clash #FIXME
+    for i in range(0, len(react_rows)):
+        for j in range(0, len(react_cols)):
+            if intrxns[i][j] is not None:
+                kf, kr = intrxns[i][j]
+                row_mpattern = react_rows[i]()
+                col_mpattern = react_cols[j]()
+                kf_parm = Parameter('bt%d%d_kf' % (i, j), kf)
+                kr_parm = Parameter('bt%d%d_kr' % (i, j), kr)
+
+                bind(react_rows[i](), row_site, react_cols[j](), col_site, [kf_parm, kr_parm])
 
 
 def two_state_equilibrium(sub, state1, state2, klist=None, sitename='loc'):
@@ -413,15 +538,16 @@ def two_state_equilibrium(sub, state1, state2, klist=None, sitename='loc'):
     Creates two rules with names following the pattern: 'monomer_state1_to_state2' and vice versa.
 
     The function generates a rule with the name following the pattern 
-    --  sub is a MonomerPattern specifying the species that translocates. The localization
-        state should not be specified here.
-    --  state1 and state2 are strings specifying the names of the locations
-        (e.g., 'c' for cytoplasmic, 'm' for mitochondrial)
-    --  klist is a list of Parameter objects specifying the forward (state1 to state2)
-        and reverse (state2 to state1) rates. If not specified, the parameters are generated
-        according to the pattern 'sub_state1_to_state2_rate' and 'sub_state2_to_state1_rate'.
-    --  sitename is an optional string specifying the name of the site that describes
-        the location. Defaults to 'loc'.
+
+    * sub is a MonomerPattern specifying the species that translocates. The localization
+      state should not be specified here.
+    * state1 and state2 are strings specifying the names of the locations
+      (e.g., 'c' for cytoplasmic, 'm' for mitochondrial)
+    * klist is a list of Parameter objects specifying the forward (state1 to state2)
+      and reverse (state2 to state1) rates. If not specified, the parameters are generated
+      according to the pattern 'sub_state1_to_state2_rate' and 'sub_state2_to_state1_rate'.
+    * sitename is an optional string specifying the name of the site that describes
+      the location. Defaults to 'loc'.
     """
 
     # FIXME: this will fail if the argument passed is a complex, or a Monomer object... 
@@ -468,15 +594,16 @@ def direct_catalysis_reversible(sub, enz, prod, klist=None):
 
     Creates two rules with names following the pattern: 'cat_sub_to_prod' and 'prod_to_sub'.
 
-    The function generates a rule with the name following the pattern 
-    --  sub is a MonomerPattern specifying the species that is acted upon.
-    --  enz is a MonomerPattern specifying the species that determines the rate of the reaction.
-        NO BINDING OCCURS BETWEEN THE SPECIES.
-    --  prod is a MonomerPattern specifying the state of the sites of sub after catalysis.
-        ANY SITES THAT ARE SPECIFIED IN SUB SHOULD BE SPECIFIED FOR PROD AND VICE VERSA.
-    --  klist is a list of Parameter objects specifying the forward (sub to prod) and
-        reverse (prod to sub) rates. If not specified, the parameters are generated
-        according to the pattern 'cat_sub_to_prod_rate' and 'prod_to_sub_rate'.
+    The function generates a rule with the name following the pattern
+
+    * sub is a MonomerPattern specifying the species that is acted upon.
+    * enz is a MonomerPattern specifying the species that determines the rate of the reaction.
+      NO BINDING OCCURS BETWEEN THE SPECIES.
+    * prod is a MonomerPattern specifying the state of the sites of sub after catalysis.
+      ANY SITES THAT ARE SPECIFIED IN SUB SHOULD BE SPECIFIED FOR PROD AND VICE VERSA.
+    * klist is a list of Parameter objects specifying the forward (sub to prod) and
+      reverse (prod to sub) rates. If not specified, the parameters are generated
+      according to the pattern 'cat_sub_to_prod_rate' and 'prod_to_sub_rate'.
     """
 
     # FIXME: this will fail if the argument passed is a complex, or a Monomer object... 
